--- /dev/null	Sun Aug 11 00:54:14 2013
+++ new/usr/src/lib/libfsd/common/libfsd.c	Sun Aug 11 00:54:14 2013
@@ -0,0 +1,176 @@
+/*
+ * This file and its contents are supplied under the terms of the
+ * Common Development and Distribution License ("CDDL"), version 1.0.
+ * You may only use this file in accordance with the terms of version
+ * 1.0 of the CDDL.
+ *
+ * A full copy of the text of the CDDL should have accompanied this
+ * source.  A copy of the CDDL is also available via the Internet at
+ * http://www.illumos.org/license/CDDL.
+ */
+
+/*
+ * Copyright 2013 Damian Bogel.  All rights reserved.
+ */
+
+/*
+ * Library used to drive the fsd pseudo-device driver.
+ */
+
+#include <fcntl.h>
+#include <libfsd.h>
+#include <string.h>
+#include <stropts.h>
+#include <sys/errno.h>
+#include <sys/fsd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
+
+__thread int fsd_errno;
+
+const char *
+fsd_strerr(int e)
+{
+	switch (e) {
+	case EFSD_NOERROR:
+		return ("no error");
+	case EFSD_BAD_STAT:
+		return ("incorrect disturbance parameters");
+	case EFSD_INTERNAL:
+		return ("internal library error");
+	case EFSD_CANT_OPEN_DRIVER:
+		return ("cannot open fsd device");
+	case EFSD_CANT_OPEN_MOUNTPOINT:
+		return ("cannot open mountpoint");
+	case EFSD_ENTRY_NOT_FOUND:
+		return ("this vfs is not being disturbed");
+	case EFSD_FAULT:
+		return ("bad pointer");
+	case EFSD_UNKNOWN_ERROR:
+	default:
+		return ("unknown error");
+	}
+}
+
+static int
+fsd_ioctl_set_errno(int ioctl_ret)
+{
+	extern int errno;
+	int e;
+
+	switch (ioctl_ret) {
+	case 0:
+		fsd_errno = EFSD_NOERROR;
+		return (0);
+
+	case (-1):
+		e = errno;
+		switch (e) {
+		case 0:
+			fsd_errno = EFSD_NOERROR;
+			break;
+		case EFAULT:
+			fsd_errno = EFSD_FAULT;
+			break;
+		case EINVAL:
+			fsd_errno = EFSD_INTERNAL;
+			break;
+		default:
+			fsd_errno = EFSD_UNKNOWN_ERROR;
+			break;
+		}
+		break;
+	case FSD_ENTRY_NOT_FOUND:
+		fsd_errno = EFSD_ENTRY_NOT_FOUND;
+		break;
+	case FSD_BAD_STAT:
+		fsd_errno = EFSD_BAD_STAT;
+		break;
+	case FSD_BAD_FD:
+		fsd_errno = EFSD_CANT_OPEN_MOUNTPOINT;
+		break;
+	default:
+		fsd_errno = EFSD_UNKNOWN_ERROR;
+		break;
+	}
+
+	return (-1);
+
+}
+
+int
+fsd_open()
+{
+	int fd;
+
+	if ((fd = open(FSD_DEV_PATH, O_RDWR)) == -1) {
+		fsd_errno = EFSD_CANT_OPEN_DRIVER;
+		return (-1);
+	}
+	return (fd);
+}
+
+void
+fsd_close(int fd)
+{
+	(void) close(fd);
+}
+
+int
+fsd_enable(int fd)
+{
+	return (fsd_ioctl_set_errno(ioctl(fd, FSD_ENABLE)));
+}
+
+int
+fsd_disable(int fd)
+{
+	return (fsd_ioctl_set_errno(ioctl(fd, FSD_DISABLE)));
+}
+
+int
+fsd_disturb(int fd, const char *mntpt_path, fsd_stat_t *stat)
+{
+	fsd_ioc_stat_t iocs;
+	int error;
+
+	if ((iocs.fsdis_mnt = open(mntpt_path, O_RDONLY)) == -1) {
+		fsd_errno = EFSD_CANT_OPEN_MOUNTPOINT;
+		return (-1);
+	}
+
+	(void) memcpy(&iocs.fsdis_stat, stat, sizeof (iocs.fsdis_stat));
+
+	error = ioctl(fd, FSD_DISTURB, &iocs);
+	(void) close(iocs.fsdis_mnt);
+	return (fsd_ioctl_set_errno(error));
+}
+
+int
+fsd_disturb_off(int fd, const char *mntpt_path)
+{
+	int error;
+	int64_t mntfd;
+
+	if ((mntfd = open(mntpt_path, O_RDONLY)) == -1) {
+		fsd_errno = EFSD_CANT_OPEN_MOUNTPOINT;
+		return (-1);
+	}
+
+	error = ioctl(fd, FSD_NODISTURB, &mntfd);
+	(void) close(mntfd);
+	return (fsd_ioctl_set_errno(error));
+}
+
+int
+fsd_newdisturb(int fd, fsd_stat_t *stat)
+{
+	return (fsd_ioctl_set_errno(ioctl(fd, FSD_DISTURB_NEW, stat)));
+}
+
+int
+fsd_newdisturb_off(int fd)
+{
+	return (fsd_ioctl_set_errno(ioctl(fd, FSD_NODISTURB_NEW)));
+}
